/* GLOGWATCH
 *  Copyright (C) 2001 Dean Johnson
 *
 *  Author:  Dean Johnson    dtj@uberh4x0r.org
 *
 *  This program is free software which I release under the GNU General Public
 *  License. You may redistribute and/or modify this program under the terms
 *  of that license as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  To get a copy of the GNU General Puplic License, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gkrellm/gkrellm.h>
#include <time.h>

extern void	insert_expanded_filler(GtkWidget *);	// gkrellm_private_proto.h

static DecalButton	*button;
static Panel	*glogwatch;
static Decal	*decal;
static Launcher	launch;
static gint	style_id;
static gboolean	glogwatch_enabled	= TRUE;
static gboolean force_redraw;

static char	theLog[1024];
static time_t	logTimestamp = ((time_t) -1);
static time_t	logMark = ((time_t) -1);

#define	GLOGWATCH_MAJOR	1	// major version
#define	GLOGWATCH_MINOR	0	// minor version

#define	SECS_IN_MINUTE	(60)
#define	SECS_IN_HOUR	(60*SECS_IN_MINUTE)
#define	SECS_IN_DAY	(24*SECS_IN_HOUR)

#define	PERIODS		1	// number of two second periods between updates

static time_t 
getTime() 
{
	struct stat	buf;
	if (stat(theLog,&buf) < 0) {
		return((time_t) -1);
	}
	return(buf.st_mtime);
}

static gint
expose_event(GtkWidget *widget, GdkEventExpose *ev)
{
	if (widget == glogwatch->drawing_area) {
		gdk_draw_pixmap(widget->window, gkrellm_draw_GC(1), glogwatch->pixmap,
				ev->area.x, ev->area.y, ev->area.x, ev->area.y,
				ev->area.width, ev->area.height);
	}
	return(FALSE);
}

static void
redraw(time_t diff)
{
	static gchar	buf[64];
	static gchar	buf2[64];
	TextStyle	*ts;
	gint	w;
	int	sec = 0;
	int	day = 0;
	int	hrs = 0;
	int	min = 0;

	ts = gkrellm_meter_textstyle(style_id);

	if (diff != 0) {
		*buf = '\0';
		sec = diff;

		day = (sec / SECS_IN_DAY);
		if (day) {
			sec -= (day * SECS_IN_DAY);
			sprintf(buf2,"%dd ",day);
			strcat(buf,buf2);
		}
		if (sec) {
			hrs = (sec / SECS_IN_HOUR);
			if (hrs) {
				sec -= (hrs * SECS_IN_HOUR);
			}
		}
		if (sec) {
			min = (sec / SECS_IN_MINUTE);
			if (min) 
				sec -= (min * SECS_IN_MINUTE);
		}
		if (day) {	// give full citation if there are days
			sprintf(buf2,"%d:%02d:%02d",hrs,min,sec);
			strcat(buf,buf2);
		}
		else	{
			if (hrs) {
				sprintf(buf2,"%d:",hrs);
				strcat(buf,buf2);
			}
			sprintf(buf2,"%02d:%02d",min,sec);
			strcat(buf,buf2);
		}
	}
	else	{
		strcpy(buf," ");
	}

	/*
	 *	Update the text.
	 */
	w = gdk_string_width(ts->font, buf);
	decal->x_off = (decal->w - w) / 2;
	if (decal->x_off < 0)
		decal->x_off = 0;
	decal->text_style.font = ts->font;
	gkrellm_draw_decal_text(glogwatch, decal, buf, -1);
	gkrellm_draw_layers(glogwatch);
}

static void
update_glogwatch(void)
{
	static int	seq = 0;

	if (!glogwatch_enabled)
		return;

	if (logTimestamp == ((time_t) -1)) {
		logTimestamp = getTime();
	}

	/* 
	 *	Once every (PERIODS * 2 seconds) is default update period.
	 */
	if (GK.two_second_tick || force_redraw) {
		if (!(seq++ % PERIODS)) {
			time_t	curTime;
			time_t	now;
			if (logMark != ((time_t) -1)) {
				/*
				 *	Just update the time, cuz you don't
				 *	need to even look at the file stat
				 *	because they haven't looked at the log.
				 */
				now = time(NULL);
				redraw(now - logMark);
				return;
			}
			else if (force_redraw)
				redraw(0);
			// update the info
			curTime = getTime();
			if (curTime > logTimestamp) {
				logMark = time(NULL);	// start timer
			}
		}
	force_redraw = FALSE;
	}
}

static void
button_pressed(DecalButton *decal)
{
        gchar	*command;
	gchar	buf[1024];

        command = g_strconcat(launch.command, " &", NULL);
	sprintf(buf,command,theLog);	//	include file in template
        system(buf);
        g_free(command);

	logMark = ((time_t) -1);	//	reset mark
	logTimestamp = getTime();	//	reset timestamp
	redraw(0);
}

static void
create_glogwatch(GtkWidget *vbox, gint first_create)
{
	static Style	style;
	TextStyle	*ts;
	gint		w;

	if (first_create) {
		glogwatch = gkrellm_panel_new0();
	}
	else	{
		gkrellm_destroy_decal_list(glogwatch);
	}
	style = *gkrellm_meter_style(style_id);
	style.label_position = LABEL_CENTER;

	glogwatch->textstyle = gkrellm_meter_textstyle(style_id);
	ts = glogwatch->textstyle;

	w = gdk_string_width(ts->font,"00:00") + 2;
	if (w > gkrellm_chart_width() - 2 * style.margin)
		w = gkrellm_chart_width() - 2 * style.margin;

	decal = gkrellm_create_decal_text(glogwatch,"00:00",ts,&style,-1,-1,w);
	decal->x = (gkrellm_chart_width() - decal->w) / 2;

	gkrellm_configure_panel(glogwatch, NULL, &style);
	gkrellm_create_panel(vbox,glogwatch,gkrellm_bg_meter_image(style_id));

	if (glogwatch_enabled)
		gkrellm_monitor_height_adjust(glogwatch->h);
	else
		gtk_widget_hide(glogwatch->hbox);

	if (first_create)
		gtk_signal_connect(GTK_OBJECT(glogwatch->drawing_area),
				   "expose_event",(GtkSignalFunc) expose_event,
				   NULL);
	button = gkrellm_put_label_in_meter_button(glogwatch,button_pressed,
						   NULL/*clientData*/,2/*pad*/);
	force_redraw = TRUE;
	update_glogwatch();
}

/*----------------------------------------------------------------------*
 *			    Configuration Stuff				*
 *----------------------------------------------------------------------*/

static void
save_config(FILE *f)
{
	fprintf(f,"glogwatch log %s\n",(*theLog) ? theLog : "localhost");
	fprintf(f,"glogwatch enable %d\n",glogwatch_enabled);
	fprintf(f,"glogwatch launch %s\n",launch.command);
	fprintf(f,"glogwatch tooltip %s\n",launch.tooltip_comment);
}

static void
load_config(gchar *arg)
{
	gchar	config[32], item[CFG_BUFSIZE];
	gint	n;

	if ((n = sscanf(arg, "%31s %[^\n]", config, item)) == 2) {
		if	(!strcmp(config, "enable"))
			sscanf(item, "%d", &glogwatch_enabled);
		else if (!strcmp(config, "log"))
			strcpy(theLog,item);
		else if (!strcmp(config, "launch"))
			launch.command = g_strdup(item);
		else if (!strcmp(config, "tooltip"))
			launch.tooltip_comment = g_strdup(item);
	}
}


static GtkWidget	*enabled_button;
static GtkWidget	*launch_entry;
static GtkWidget	*tooltip_entry;
static GtkWidget	*log_entry;

static void
apply_config()
{
	gboolean	new;
	char	*txt;

	new = GTK_TOGGLE_BUTTON(enabled_button)->active;
	gkrellm_enable_visibility(new,&glogwatch_enabled,glogwatch->hbox, 
				  glogwatch->h);
	txt = gtk_entry_get_text(GTK_ENTRY(log_entry));
	if (txt && *txt)
		strcpy(theLog,txt);
	txt = gtk_entry_get_text(GTK_ENTRY(launch_entry));
	if (txt && *txt)
		launch.command = strdup(txt);
	txt = gtk_entry_get_text(GTK_ENTRY(tooltip_entry));
	if (txt && *txt)
		launch.tooltip_comment = strdup(txt);
        gkrellm_configure_tooltip(glogwatch,&launch);
}

static void
create_text_config_entry(GtkWidget *box, GtkWidget **entry, gchar *defaultVal,
		    	 gint width,gpointer data,gchar *string)
{
	GtkWidget	*hbox	= NULL;
	GtkWidget	*label;

	hbox = gtk_hbox_new (FALSE, 2);
	gtk_box_pack_start(GTK_BOX(box), hbox, TRUE, TRUE, 0);
	box = hbox;

	label = gtk_label_new(string);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(box), label, FALSE, TRUE, 2);

	*entry = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(box), *entry, TRUE, TRUE, 0);
	if (width > 0)
		gtk_widget_set_usize(*entry, width, 0);
	if (defaultVal)
		gtk_entry_set_text(GTK_ENTRY(*entry),defaultVal);
}

static void
tab_glogwatch(GtkWidget *tab_vbox)
{
	GtkWidget	*table;
	GtkWidget	*label;
	gchar	*buf;

	/*
	 *	Self-promoting about stuff.
	 */

	label = gtk_label_new("");
	gtk_box_pack_start(GTK_BOX(tab_vbox), label, TRUE, TRUE, 0);

	buf = g_strdup_printf(_("GLOGWATCH %d.%d - GKrellM LOGWATCH Plugin\n\n"
				"Copyright (c) 2001 by Dean Johnson\n"
				"dtj@uberh4x0r.org\n\n"
				"Released under the GNU Public License"),
				GLOGWATCH_MAJOR,GLOGWATCH_MINOR);
	label = gtk_label_new(buf);
	g_free(buf);
	gtk_box_pack_start(GTK_BOX(tab_vbox), label, FALSE, FALSE, 0);
	label = gtk_label_new("");
	gtk_box_pack_start(GTK_BOX(tab_vbox), label, TRUE, TRUE, 0);

	/*
	 *	Actual Meat
	 */

	gkrellm_check_button(tab_vbox, &enabled_button,glogwatch_enabled,FALSE,
			     10,_("Enable GLOGWATCH"));
	create_text_config_entry(tab_vbox, &log_entry, theLog,100,NULL,
				 _("GLOGWATCH file"));
	insert_expanded_filler(tab_vbox);
	table = gkrellm_launcher_table_new(tab_vbox, 1);
	gkrellm_config_launcher(table,0,&launch_entry,&tooltip_entry,
				_("GLOGWATCH"), &launch);
}

static Monitor	monitor_glogwatch = {
	N_("GLOGWATCH"),	/* Config tab name		*/
	0,			/* Id,  0 if a plugin		*/
	create_glogwatch,	/* create function		*/
	update_glogwatch,	/* update function		*/
	tab_glogwatch,		/* config tab create function	*/
	apply_config,		/* Apply the config function	*/
	save_config,		/* Save user conifg		*/
	load_config,		/* Load user config		*/
	"glogwatch",		/* config keyword		*/
	NULL,
	NULL,
	NULL,
	0,			/* insert_before_id		*/
	NULL,			/* plugin Handle		*/
	NULL			/* plugin path			*/
};

Monitor *
init_plugin(void)
{
	strcpy(theLog,"/var/log/messages");
	launch.command		= strdup("xterm -e vi + %s");
	launch.tooltip_comment	= strdup(" ");
	monitor_glogwatch.name	= _(monitor_glogwatch.name);
	style_id = gkrellm_add_meter_style(&monitor_glogwatch,"glogwatch");
	return(&monitor_glogwatch);
}
